<?php
/*
* $Id: bm_recently_viewed.php
* $Loc: /includes/modules/boxes/
*
* Name: RecentlyViewed
* Version: 5.3.9
* Release Date: 2024-10-22
* Author: Rainer Schmied
* 	 phoenixcartaddonsaddons.com / raiwa@phoenixcartaddons.com
*
* License: Released under the GNU General Public License
*
* Comments: Author: [Rainer Schmied @raiwa]
* Author URI: [www.phoenixcartaddons.com]
* 
* CE Phoenix, E-Commerce made Easy
* https://phoenixcart.org
* 
* Copyright (c) 2021 Phoenix Cart
* 
* 
*/


  class bm_recently_viewed extends abstract_block_module {

    const CONFIG_KEY_BASE = 'MODULE_BOXES_RECENTLY_VIEWED_';

    protected $group = 'boxes';

    public function __construct() {
      parent::__construct(__FILE__);

      $this->description .= '<p>by @raiwa <u><a target="_blank" href="http://www.oscaddons.com">www.oscaddons.com</a></u></p>';

      if (!file_exists(DIR_FS_CATALOG . 'templates/' . TEMPLATE_SELECTION . '/includes/hooks/shop/siteWide/recentlyViewed.php')) {
        $this->description = '<div class="alert alert-danger">' . MODULE_BOXES_RECENTLY_VIEWED_ERROR_HOOK . '</div>' . $this->description;
        $this->enabled = false;
      }
    }

    function execute() {

      $recently_viewed = $_SESSION['recently_viewed'] ?? null;

      if (!empty($recently_viewed)) {
        $recently_products_query_where = '';
        if ( MODULE_BOXES_RECENTLY_VIEWED_SHOW_CURRENT == 'False' && isset($_GET['products_id']) ) {
           $recently_products_query_where .= " AND products_id != '" . (int)$_GET['products_id'] . "'";
        }

        if (MODULE_BOXES_RECENTLY_VIEWED_DISPLAY_ORDER == 'Newest') {
           $recently_products_query_order = " ORDER BY FIELD(products_id," . $recently_viewed . ") ASC";
        } elseif (MODULE_BOXES_RECENTLY_VIEWED_DISPLAY_ORDER == 'Oldest') {
           $recently_products_query_order = " ORDER BY FIELD(products_id," . $recently_viewed . ") DESC";
        } else {
           $recently_products_query_order = " ORDER BY rand()";
        }

        $recently_products_query = $GLOBALS['db']->query(sprintf(<<<'EOSQL'
SELECT products_id
  FROM products
  WHERE products_id IN (%s)
    AND products_status = 1
    %s
    %s
  LIMIT %s
EOSQL
        , $recently_viewed, $recently_products_query_where, $recently_products_query_order, (int)MODULE_BOXES_RECENTLY_VIEWED_MAX_DISPLAY_PRODUCTS));

        $num_rows = mysqli_num_rows($recently_products_query);
        if (!$num_rows) {
          return;
        }

        if ($num_rows > 1) {
          $recently_products_query->data_seek(mt_rand(0, $num_rows - 1));
        }
        $recently_products_selection = $recently_products_query->fetch_assoc();

        $product = product_by_id::build((int)$recently_products_selection['products_id']);

        $box = [
          'parameters' => ['product_card.php', 'component'],
          'classes' => 'is-product bm-whats-new',
          'title' => sprintf(MODULE_BOXES_RECENTLY_VIEWED_BOX_TITLE, $GLOBALS['Linker']->build('recently_viewed.php')),
          'attributes' => $product->get('data_attributes'),
        ];

        $tpl_data = [
          'group' => $this->group,
          'file' => 'box.php',
          'type' => 'component',
        ];
        include 'includes/modules/block_template.php';
      }
    }

    protected function get_parameters() {
      return [
        'MODULE_BOXES_RECENTLY_VIEWED_VERSION_INSTALLED' => [
          'title' => 'Current Version',
          'value' => '5.3.4',
          'desc' => 'Version info. It is read only',
          'set_func' => 'bm_recently_viewed::readonly(',
        ],
        'MODULE_BOXES_RECENTLY_VIEWED_STATUS' => [
          'title' => 'Enable Recently Viewed Box',
          'value' => 'True',
          'desc' => 'Do you want to enable this module?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        'MODULE_BOXES_RECENTLY_VIEWED_CONTENT_PLACEMENT' => [
          'title' => 'Content Placement',
          'value' => 'Right Column',
          'desc' => 'Should the module be loaded in the left or right column?',
          'set_func' => "Config::select_one(['Left Column', 'Right Column'], ",
        ],
        'MODULE_BOXES_RECENTLY_VIEWED_SORT_ORDER' => [
          'title' => 'Sort Order',
          'value' => '5100',
          'desc' => 'Sort order of display. Lowest is displayed first.',
        ],
        'MODULE_BOXES_RECENTLY_VIEWED_MAX_DISPLAY_PRODUCTS' => [
          'title' => 'Number of Products',
          'value' => '5',
          'desc' => 'Maximum number of products to display in this box.',
        ],
        'MODULE_BOXES_RECENTLY_VIEWED_SHOW_DESCRIPTION' => [
          'title' => 'Show the product description',
          'value' => 'False',
          'desc' => 'Show the product description in this box?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        'MODULE_BOXES_RECENTLY_VIEWED_DESCRIPTION_LENGTH' => [
          'title' => 'Maximum description length',
          'value' => '25',
          'desc' => 'The number of words of the description to display in the recently viewed box',
        ],
        'MODULE_BOXES_RECENTLY_VIEWED_SHOW_MORE_BUTTON' => [
          'title' => 'Show the product MORE button',
          'value' => 'True',
          'desc' => 'Show the MORE button in this box?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        'MODULE_BOXES_RECENTLY_VIEWED_SHOW_CURRENT' => [
          'title' => 'Show the current product',
          'value' => 'False',
          'desc' => 'Show the current product in the Recently Viewed box?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        'MODULE_BOXES_RECENTLY_VIEWED_DISPLAY_ORDER' => [
          'title' => 'Product order',
          'value' => 'Newest',
          'desc' => 'Show the newest or oldest recently viewed product at the beginning of the Recently Viewed box? Or use random order.',
          'set_func' => "Config::select_one(['Newest', 'Oldest', 'Random'], ",
        ],
      ];
    }

    public static function readonly($value) {
      return $value;
    }

  }
