<?php
/*
* $Id: cm_header_recently_viewed.php
* $Loc: /includes/modules/content/header/
*
* Name: RecentlyViewed
* Version: 5.3.9
* Release Date: 2024-10-22
* Author: Rainer Schmied
* 	 phoenixcartaddonsaddons.com / raiwa@phoenixcartaddons.com
*
* License: Released under the GNU General Public License
*
* Comments: Author: [Rainer Schmied @raiwa]
* Author URI: [www.phoenixcartaddons.com]
* 
* CE Phoenix, E-Commerce made Easy
* https://phoenixcart.org
* 
* Copyright (c) 2021 Phoenix Cart
* 
* 
*/

  class cm_header_recently_viewed extends abstract_executable_module {

    const CONFIG_KEY_BASE = 'MODULE_CONTENT_HEADER_RECENTLY_VIEWED_';

    public function __construct() {
      parent::__construct(__FILE__);

      $this->description .= '<p>by @raiwa <u><a target="_blank" href="http://www.oscaddons.com">www.oscaddons.com</a></u></p>';

      if (!file_exists(DIR_FS_CATALOG . 'templates/' . TEMPLATE_SELECTION . '/includes/hooks/shop/siteWide/recentlyViewed.php')) {
        $this->description = '<div class="alert alert-danger">' . MODULE_CONTENT_HEADER_RECENTLY_VIEWED_ERROR_HOOK . '</div>' . $this->description;
        $this->enabled = false;
      }
    }

    function execute() {

      $recently_viewed = $_SESSION['recently_viewed'] ?? null;

      if (!empty($recently_viewed)) {

        $recently_products_query_where = '';
        if ( MODULE_CONTENT_HEADER_RECENTLY_VIEWED_SHOW_CURRENT == 'False' && isset($_GET['products_id']) ) {
          $recently_products_query_where .= " AND products_id != '" . (int)$_GET['products_id'] . "'";
        }

        if (MODULE_CONTENT_HEADER_RECENTLY_VIEWED_DISPLAY_ORDER == 'Newest') {
           $recently_products_query_order = " FIELD(products_id," . $recently_viewed . ") ASC";
        } elseif (MODULE_CONTENT_HEADER_RECENTLY_VIEWED_DISPLAY_ORDER == 'Oldest') {
           $recently_products_query_order = " FIELD(products_id," . $recently_viewed . ") DESC";
        } else {
           $recently_products_query_order = " rand()";
        }

        $recently_products_sql = sprintf(<<<'EOSQL'
SELECT DISTINCT products_id
  FROM products
  WHERE products_id IN (%s)
    AND products_status = 1
EOSQL
      , $recently_viewed);

        if ( MODULE_CONTENT_HEADER_RECENTLY_VIEWED_SHOW_CURRENT == 'False' && isset($_GET['products_id']) ) {
          $recently_products_sql .= sprintf(<<<'EOSQL'
    AND products_id != %s
EOSQL
          , (int)$_GET['products_id']);
        }
        
        $recently_products_sql .= sprintf(<<<'EOSQL'
  ORDER BY %s
  LIMIT %s
EOSQL
      , $recently_products_query_order, (int)MODULE_CONTENT_HEADER_RECENTLY_VIEWED_CONTENT_LIMIT);

        $recently_products_query = $GLOBALS['db']->query($recently_products_sql);
        $num_rows = mysqli_num_rows($recently_products_query);

        if (!$num_rows) {
          return;
        }

        while ( $recently_products = $recently_products_query->fetch_assoc() ) {
          $product = product_by_id::build((int)$recently_products['products_id']);

          $linkArray[] = '<button class="dropdown-item pr-5" type="button"><a href="' . $product->get('link') .'">';

          if (MODULE_CONTENT_HEADER_RECENTLY_VIEWED_SHOW_IMAGE == 'True')
            $linkArray[] .= new Image('images/' . $product->get('image'), ['width' => '40', 'height' =>'40'], htmlspecialchars($product->get('name')));

          if (MODULE_CONTENT_HEADER_RECENTLY_VIEWED_SHOW_NAME == 'True')
            $linkArray[] .= $this->_truncate_text_rel_prod($product->get('name'), MODULE_CONTENT_HEADER_RECENTLY_VIEWED_NAME_LENGTH, MODULE_CONTENT_HEADER_RECENTLY_VIEWED_WORD_LENGTH);

          if (MODULE_CONTENT_HEADER_RECENTLY_VIEWED_SHOW_PRICE == 'True') {
            $linkArray[] .=  '&nbsp;' . $product->hype_price();
          }

          $linkArray[] = '</a></button>';
        }

        $tpl_data = [ 'group' => $this->group, 'file' => __FILE__ ];
        include 'includes/modules/content/cm_template.php';
      }
    }

    protected function get_parameters() {
      return [
        $this->config_key_base . 'VERSION_INSTALLED' => [
          'title' => 'Current Version',
          'value' => '5.3.8',
          'desc' => 'Version info. It is read only',
          'set_func' => 'cm_header_recently_viewed::readonly(',
        ],
        $this->config_key_base . 'STATUS' => [
          'title' => 'Enable Recently Viewed Module',
          'value' => 'True',
          'desc' => 'Do you want to enable this module?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        $this->config_key_base . 'CONTENT_WIDTH' => [
          'title' => 'Content Container',
          'value' => 'col-sm-12 mb-1 px-0',
          'desc' => 'What container should the content be shown in? (col-*-12 = full width, col-*-6 = half width).',
        ],
        $this->config_key_base . 'SORT_ORDER' => [
          'title' => 'Sort Order',
          'value' => '1200',
          'desc' => 'Sort order of display. Lowest is displayed first.',
        ],
        $this->config_key_base . 'CONTENT_LIMIT' => [
          'title' => 'Number of Products',
          'value' => '6',
          'desc' => 'Maximum Number of products that should show in this module?',
        ],
        $this->config_key_base . 'SHOW_IMAGE' => [
          'title' => 'Show the product image',
          'value' => 'True',
          'desc' => 'Show the product image in this module?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        $this->config_key_base . 'SHOW_NAME' => [
          'title' => 'Show the product name',
          'value' => 'True',
          'desc' => 'Show the product name in this module?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        $this->config_key_base . 'SHOW_PRICE' => [
          'title' => 'Show the product price',
          'value' => 'True',
          'desc' => 'Show the product price in this module?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        $this->config_key_base . 'WORD_LENGTH' => [
          'title' => 'Maximum word length',
          'value' => '40',
          'desc' => 'Maximum number of characters in a single word (Needed to prevent breaking menu width)',
        ],
        $this->config_key_base . 'NAME_LENGTH' => [
          'title' => 'Maximum of characters',
          'value' => '32',
          'desc' => 'Maximum number of characters of the product name to display in the recently_viewed module (to the nearest word)?',
        ],
        $this->config_key_base . 'SHOW_MORE_BUTTON' => [
          'title' => 'Show the product MORE button',
          'value' => 'True',
          'desc' => 'Show the MORE button in this module?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        $this->config_key_base . 'SHOW_CURRENT' => [
          'title' => 'Show the current product',
          'value' => 'False',
          'desc' => 'Show the current product in the Recently Viewed module?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        $this->config_key_base . 'DISPLAY_ORDER' => [
          'title' => 'Product order',
          'value' => 'Newest',
          'desc' => 'Show the newest or oldest recently viewed product at the beginning of the Recently Viewed module? Or use random order.',
          'set_func' => "Config::select_one(['Newest', 'Oldest', 'Random'], ",
        ],
      ];
    }

    protected function _limit_text_rel_prod ($text, $maxchar, $wordlength = 40) {
      $text = str_replace ("\n", ' ', $text);
      $text = str_replace ("\r", ' ', $text);
      $text = str_replace ('<br>', ' ', $text);
      $text = wordwrap ($text, $wordlength, ' ', true);
      $text = preg_replace ("/[ ]+/", ' ', $text);
      $text_length = strlen ($text);
      $text_array = explode (" ", $text);

      $newtext = '';
      for ($array_key = 0, $length = 0; $length <= $text_length; $array_key++) {
        $length = strlen ($newtext) + strlen ($text_array[$array_key]) + 1;
        if ($length > $maxchar) break;
        $newtext = $newtext . ' ' . $text_array[$array_key];
      }

      return $newtext;
    }

    protected function _truncate_text_rel_prod ($products_text, $maxchar, $wordlength = 40) {
      $products_text = ($products_text);
      if ($maxchar > 0) {
        $products_text_length = strlen ($products_text);
        if ($products_text_length > $maxchar) {
          $products_text = $this->_limit_text_rel_prod ($products_text, $maxchar, $wordlength);
          $products_text .= '&nbsp;...';
        }
      }

      return $products_text;
    }

    public static function readonly($value) {
      return $value;
    }

  }
