<?php
/*
* $Id: cm_sc_continue_recently_viewed.php
* $Loc: /includes/modules/content/shopping_cart/
*
* Name: RecentlyViewed
* Version: 5.3.9
* Release Date: 2024-10-22
* Author: Rainer Schmied
* 	 phoenixcartaddonsaddons.com / raiwa@phoenixcartaddons.com
*
* License: Released under the GNU General Public License
*
* Comments: Author: [Rainer Schmied @raiwa]
* Author URI: [www.phoenixcartaddons.com]
* 
* CE Phoenix, E-Commerce made Easy
* https://phoenixcart.org
* 
* Copyright (c) 2021 Phoenix Cart
* 
* 
*/

  class cm_sc_continue_recently_viewed extends abstract_executable_module {

    const CONFIG_KEY_BASE = 'MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_';

    public function __construct() {
      parent::__construct(__FILE__);

      $this->description .= '<p>by @raiwa <u><a target="_blank" href="http://www.oscaddons.com">www.oscaddons.com</a></u></p>';

      if (!file_exists(DIR_FS_CATALOG . 'templates/' . TEMPLATE_SELECTION . '/includes/hooks/shop/siteWide/recentlyViewed.php')) {
        $this->description = '<div class="alert alert-danger">' . MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_ERROR_HOOK . '</div>' . $this->description;
        $this->enabled = false;
      }
    }

    function execute() {

      $recently_viewed = $_SESSION['recently_viewed'] ?? null;

      if (!empty($recently_viewed)) {
        $ids_in_cart_list = null;

        $products = $_SESSION['cart']->get_products();
        if ( $_SESSION['cart']->count_contents() !== 0 ) {
          foreach ($products as $product) {
            $ids_in_cart_array[] = $product->get('id');
          }
          $ids_in_cart_list = implode(',', $ids_in_cart_array);
        }

        if (MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_DISPLAY_ORDER == 'Newest') {
          $products_query_order = " FIELD(p.products_id," . $recently_viewed . ") ASC";
        } elseif (MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_DISPLAY_ORDER == 'Oldest') {
          $products_query_order = " FIELD(p.products_id," . $recently_viewed . ") DESC";
        } else {
          $products_query_order = " rand()";
        }

        $recently_products_sql = sprintf(<<<'EOSQL'
SELECT DISTINCT %s
EOSQL
        , Product::COLUMNS);

        $recently_products_sql .= <<<'EOSQL'
  FROM products p
    LEFT JOIN specials s ON p.products_id = s.products_id
    INNER JOIN products_description pd ON p.products_id = pd.products_id
    LEFT JOIN (SELECT products_id, COUNT(*) AS attribute_count FROM products_attributes GROUP BY products_id) a ON p.products_id = a.products_id
  WHERE p.products_status = 1
EOSQL
        ;

        if ( MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_SHOW_CURRENT == 'False'  && !empty($ids_in_cart_list) ) {
          $recently_products_sql .= sprintf(<<<'EOSQL'
     AND p.products_id NOT IN (%s)
EOSQL
          , Text::input($ids_in_cart_list));
        }

        $recently_products_sql .= sprintf(<<<'EOSQL'
    AND p.products_id IN (%s)
    AND pd.language_id = %d
  ORDER BY %s
  LIMIT %d
EOSQL
        , Text::input($recently_viewed),
          (int)$_SESSION['languages_id'],
          $products_query_order,
          (int)MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_CONTENT_LIMIT);

        $recently_products_query = $GLOBALS['db']->query($recently_products_sql);

        if (mysqli_num_rows($recently_products_query) > 0) {
          while ( $recently_products = $recently_products_query->fetch_assoc() ) {
            $product = product_by_id::build((int)$recently_products['products_id']);

            $linkArray[] = '<button class="dropdown-item" type="button"><a href="' . $product->get('link') .'">';

            if (MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_SHOW_IMAGE == 'True')
              $linkArray[] .= new Image('images/' . $product->get('image'), ['width' => '40', 'height' =>'40'], htmlspecialchars($product->get('name')));

            $linkArray[] .= $this->_truncate_text_rel_prod($product->get('name'), MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_NAME_LENGTH, MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_WORD_LENGTH);

            if (MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_SHOW_PRICE == 'True') {
              $linkArray[] .=  '&nbsp;' . $product->hype_price();
            }

            $linkArray[] = '</a></button>';
          }

          $products = $_SESSION['cart']->get_products();
          $back = sizeof($_SESSION['navigation']->path)-2;
          $continueButtonId = null;
          if ( $_SESSION['cart']->count_contents() !== 0 ) {
            foreach ($products as $product) {
              $continueButtonId = product_by_id::build($product->get('id'))->find_path();
            }
          }

          if ( MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_CONTINUE_BUTTON == 'True' ) {
            if ( !empty($continueButtonId) ) {
              $backlink = Guarantor::ensure_global('Linker')->build('index.php', 'cPath=' . $continueButtonId);
            } elseif( isset($_SERVER['HTTP_REFERER']) && (strpos($_SERVER['HTTP_REFERER'], 'checkout' ) !== false) ) {
              $backlink = Guarantor::ensure_global('Linker')->build('index.php');
            } elseif (isset($_SESSION['navigation']->path[$back])) {
              $back_path = '';
              foreach ($_SESSION['navigation']->path[$back]['get'] as $key => $value ) {
                $back_path = $key . '=' . $value;
              }
              $backlink = Guarantor::ensure_global('Linker')->build($_SESSION['navigation']->path[$back]['page'],  $back_path);
            } else {
              $backlink = Guarantor::ensure_global('Linker')->build('index.php');
            }
          }

          $tpl_data = [ 'group' => $this->group, 'file' => __FILE__ ];
          include 'includes/modules/content/cm_template.php';
        }
      }
    }

    protected function get_parameters() {
      return [
        'MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_VERSION_INSTALLED' => [
          'title' => 'Current Version',
          'value' => '5.3.6',
          'desc' => 'Version info. It is read only',
          'set_func' => 'cm_sc_continue_recently_viewed::readonly(',
        ],
        'MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_STATUS' => [
          'title' => 'Enable Continue shopping & Recently menu',
          'value' => 'True',
          'desc' => 'Do you want to enable this module?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        'MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_CONTENT_WIDTH' => [
          'title' => 'Content Width',
          'value' => '12',
          'desc' => 'What width container should the content be shown in? (12 = full width, 6 = half width).',
          'set_func' => "Config::select_one(['12', '11', '10', '9', '8', '7', '6', '5', '4', '3', '2', '1'], ",
        ],
        'MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_SORT_ORDER' => [
          'title' => 'Sort Order',
          'value' => '1200',
          'desc' => 'Sort order of display. Lowest is displayed first.',
        ],
        'MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_CONTINUE_BUTTON' => [
          'title' => 'Show Continue Shopping Button',
          'value' => 'True',
          'desc' => 'Should the Continue Shopping Button be shown on the shopping cart page?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        'MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_CONTENT_LIMIT' => [
          'title' => 'Number of Products',
          'value' => '6',
          'desc' => 'Maximum Number of products that should show in this module?',
        ],
        'MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_SHOW_IMAGE' => [
          'title' => 'Show the product image',
          'value' => 'True',
          'desc' => 'Show the product image in this module?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        'MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_SHOW_PRICE' => [
          'title' => 'Show the product price',
          'value' => 'True',
          'desc' => 'Show the product price in this module?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        'MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_WORD_LENGTH' => [
          'title' => 'Maximum word length',
          'value' => '40',
          'desc' => 'Maximum number of characters in a single word (Needed to prevent breaking menu width)',
        ],
        'MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_NAME_LENGTH' => [
          'title' => 'Maximum of characters',
          'value' => '32',
          'desc' => 'Maximum number of characters of the product name to display in the recently_viewed module (to the nearest word)?',
        ],
        'MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_SHOW_MORE_BUTTON' => [
          'title' => 'Show the product MORE button',
          'value' => 'True',
          'desc' => 'Show the MORE button in this module?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        'MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_SHOW_CURRENT' => [
          'title' => 'Show the current product',
          'value' => 'False',
          'desc' => 'Show the current product in the Recently Viewed module?',
          'set_func' => "Config::select_one(['True', 'False'], ",
        ],
        'MODULE_CONTENT_SC_CONTINUE_RECENTLY_VIEWED_DISPLAY_ORDER' => [
          'title' => 'Product order',
          'value' => 'Newest',
          'desc' => 'Show the newest or oldest recently viewed product at the beginning of the Recently Viewed module? Or use random order.',
          'set_func' => "Config::select_one(['Newest', 'Oldest', 'Random'], ",
        ],
      ];
    }

    protected function _limit_text_rel_prod ($text, $maxchar, $wordlength = 40) {
      $text = str_replace ("\n", ' ', $text);
      $text = str_replace ("\r", ' ', $text);
      $text = str_replace ('<br>', ' ', $text);
      $text = wordwrap ($text, $wordlength, ' ', true);
      $text = preg_replace ("/[ ]+/", ' ', $text);
      $text_length = strlen ($text);
      $text_array = explode (" ", $text);

      $newtext = '';
      for ($array_key = 0, $length = 0; $length <= $text_length; $array_key++) {
        $length = strlen ($newtext) + strlen ($text_array[$array_key]) + 1;
        if ($length > $maxchar) break;
        $newtext = $newtext . ' ' . $text_array[$array_key];
      }

      return $newtext;
    }

    protected function _truncate_text_rel_prod ($products_text, $maxchar, $wordlength = 40) {
      $products_text = ($products_text);
      if ($maxchar > 0) {
        $products_text_length = strlen ($products_text);
        if ($products_text_length > $maxchar) {
          $products_text = $this->_limit_text_rel_prod ($products_text, $maxchar, $wordlength);
          $products_text .= '&nbsp;...';
        }
      }

      return $products_text;
   }

    public static function readonly($value) {
      return $value;
    }

  }
